import enum
from dataclasses import dataclass
from typing import Dict, Optional, Union

from feast.core.DataSource_pb2 import DataSource as DataSourceProto
from feast.core.SparkOverride_pb2 import SparkOverride as SparkOverrideProto
from feast.data_format import FileFormat, StreamFormat


class SourceType(enum.Enum):
    """
    DataSource value type. Used to define source types in DataSource.
    """

    UNKNOWN = 0
    BATCH_FILE = 1
    BATCH_BIGQUERY = 2


@dataclass
class SparkOverride:
    """
    Overridable spark configuration for ingestion jobs.
    """

    driver_cpu: int = 0
    executor_cpu: int = 0
    driver_memory: str = ""
    executor_memory: str = ""

    @classmethod
    def from_proto(cls, spark_override_proto: SparkOverrideProto):
        """
        Create a new instance from the corresponding protobuf representation.
        Args:
            spark_override_proto (object):
        Returns:
            A new instance of the class.
        """
        return cls(
            driver_cpu=spark_override_proto.driver_cpu,
            executor_cpu=spark_override_proto.executor_cpu,
            driver_memory=spark_override_proto.driver_memory,
            executor_memory=spark_override_proto.executor_memory,
        )

    def to_proto(self) -> SparkOverrideProto:
        """
        Returns:
            Protobuf representation of the instance.
        """
        return SparkOverrideProto(
            driver_cpu=self.driver_cpu,
            executor_cpu=self.executor_cpu,
            driver_memory=self.driver_memory,
            executor_memory=self.executor_memory,
        )


@dataclass
class FileOptions:
    """
    DataSource File options used to source features from a file
    """
    file_format: FileFormat
    file_url: str
    spark_override: Optional[SparkOverride] = None

    @classmethod
    def from_proto(cls, file_options_proto: DataSourceProto.FileOptions):
        """
        Create a new instance from the corresponding protobuf representation.
        args:
            file_options_proto: a protobuf representation of a datasource
        Returns:
            A new instance of the class.
        """
        return cls(
            file_format=FileFormat.from_proto(file_options_proto.file_format),
            file_url=file_options_proto.file_url,
            spark_override=SparkOverride.from_proto(file_options_proto.spark_override)
        )

    def to_proto(self) -> DataSourceProto.FileOptions:
        """
        Returns:
            Protobuf representation of the instance.
        """

        return DataSourceProto.FileOptions(
            file_format=self.file_format.to_proto(),
            file_url=self.file_url,
            spark_override=self.spark_override.to_proto() if self.spark_override else None,
        )


@dataclass
class BigQueryOptions:
    """
    DataSource BigQuery options used to source features from BigQuery query
    """
    table_ref: str
    spark_override: Optional[SparkOverride] = None

    @classmethod
    def from_proto(cls, bigquery_options_proto: DataSourceProto.BigQueryOptions):
        """
        Creates a BigQueryOptions from a protobuf representation of a BigQuery option
        Args:
            bigquery_options_proto: A protobuf representation of a DataSource
        Returns:
            Returns a BigQueryOptions object based on the bigquery_options protobuf
        """

        return cls(
            table_ref=bigquery_options_proto.table_ref,
            spark_override=SparkOverride.from_proto(bigquery_options_proto.spark_override),
        )

    def to_proto(self) -> DataSourceProto.BigQueryOptions:
        """
        Converts an BigQueryOptionsProto object to its protobuf representation.
        Returns:
            BigQueryOptionsProto protobuf
        """

        return DataSourceProto.BigQueryOptions(
            table_ref=self.table_ref,
            spark_override=self.spark_override.to_proto() if self.spark_override else None,
        )


@dataclass
class FileSource:
    event_timestamp_column: str
    file_format: FileFormat
    file_url: str
    created_timestamp_column: str = ""
    field_mapping: Optional[Dict[str, str]] = None
    date_partition_column: str = ""
    spark_override: Optional[SparkOverride] = None

    def to_proto(self) -> DataSourceProto:
        return DataSourceProto(
            type=DataSourceProto.BATCH_FILE,
            field_mapping=self.field_mapping,
            file_options=FileOptions(file_format=self.file_format, file_url=self.file_url,
                                     spark_override=self.spark_override).to_proto() if self.spark_override else None,
            event_timestamp_column=self.event_timestamp_column,
            created_timestamp_column=self.created_timestamp_column,
            date_partition_column=self.date_partition_column,
        )


@dataclass
class BigQuerySource:
    event_timestamp_column: str
    table_ref: str
    created_timestamp_column: str = ""
    field_mapping: Optional[Dict[str, str]] = None
    date_partition_column: str = ""
    spark_override: Optional[SparkOverride] = None

    def to_proto(self) -> DataSourceProto:
        return DataSourceProto(
            type=DataSourceProto.BATCH_BIGQUERY,
            field_mapping=self.field_mapping,
            bigquery_options=BigQueryOptions(self.table_ref, self.spark_override).to_proto(),
            event_timestamp_column=self.event_timestamp_column,
            created_timestamp_column=self.created_timestamp_column,
            date_partition_column=self.date_partition_column,
        )


@dataclass
class KafkaOptions:
    """
    DataSource Kafka options used to source features from Kafka messages
    """
    bootstrap_servers: str
    message_format: StreamFormat
    topic: str
    spark_override: Optional[SparkOverride] = None

    @classmethod
    def from_proto(cls, kafka_options_proto: DataSourceProto.KafkaOptions):
        """
        Creates a KafkaOptions from a protobuf representation of a kafka option
        Args:
            kafka_options_proto: A protobuf representation of a DataSource
        Returns:
            Returns a BigQueryOptions object based on the kafka_options protobuf
        """

        return cls(
            bootstrap_servers=kafka_options_proto.bootstrap_servers,
            message_format=StreamFormat.from_proto(kafka_options_proto.message_format),
            topic=kafka_options_proto.topic,
            spark_override=SparkOverride.from_proto(kafka_options_proto.spark_override),
        )

    def to_proto(self) -> DataSourceProto.KafkaOptions:
        """
        Converts an KafkaOptionsProto object to its protobuf representation.
        Returns:
            KafkaOptionsProto protobuf
        """

        return DataSourceProto.KafkaOptions(
            bootstrap_servers=self.bootstrap_servers,
            message_format=self.message_format.to_proto(),
            topic=self.topic,
            spark_override=self.spark_override.to_proto() if self.spark_override else None,
        )


@dataclass
class KafkaSource:
    event_timestamp_column: str
    bootstrap_servers: str
    message_format: StreamFormat
    topic: str
    created_timestamp_column: str = ""
    field_mapping: Optional[Dict[str, str]] = None
    date_partition_column: str = ""
    spark_override: Optional[SparkOverride] = None

    def to_proto(self) -> DataSourceProto:
        return DataSourceProto(
            type=DataSourceProto.STREAM_KAFKA,
            field_mapping=self.field_mapping,
            kafka_options=KafkaOptions(bootstrap_servers=self.bootstrap_servers,
                                       message_format=self.message_format, topic=self.topic, spark_override=self.spark_override).to_proto(),
            event_timestamp_column=self.event_timestamp_column,
            created_timestamp_column=self.created_timestamp_column,
            date_partition_column=self.date_partition_column,
        )


def new_batch_source_from_proto(data_source: DataSourceProto) -> Union[BigQuerySource, FileSource]:
    """
    Convert data source config in FeatureTable spec proto to one of the data source model.
    """

    if data_source.file_options.file_format and data_source.file_options.file_url:
        return FileSource(
            field_mapping=dict(data_source.field_mapping),
            file_format=FileFormat.from_proto(data_source.file_options.file_format),
            file_url=data_source.file_options.file_url,
            event_timestamp_column=data_source.event_timestamp_column,
            created_timestamp_column=data_source.created_timestamp_column,
            date_partition_column=data_source.date_partition_column,
            spark_override=SparkOverride.from_proto(data_source.file_options.spark_override),
        )
    elif data_source.bigquery_options.table_ref:
        return BigQuerySource(
            field_mapping=dict(data_source.field_mapping),
            table_ref=data_source.bigquery_options.table_ref,
            event_timestamp_column=data_source.event_timestamp_column,
            created_timestamp_column=data_source.created_timestamp_column,
            date_partition_column=data_source.date_partition_column,
            spark_override=SparkOverride.from_proto(data_source.bigquery_options.spark_override),
        )
    else:
        raise ValueError("Could not identify the source type being added")


def new_stream_source_from_proto(data_source: DataSourceProto) -> KafkaSource:
    """
    Convert data source config in FeatureTable spec proto to one of the data source model.
    """
    if data_source.kafka_options.topic:
        return KafkaSource(
            field_mapping=dict(data_source.field_mapping),
            bootstrap_servers=data_source.kafka_options.bootstrap_servers,
            message_format=StreamFormat.from_proto(
                data_source.kafka_options.message_format
            ),
            topic=data_source.kafka_options.topic,
            event_timestamp_column=data_source.event_timestamp_column,
            created_timestamp_column=data_source.created_timestamp_column,
            date_partition_column=data_source.date_partition_column,
            spark_override=SparkOverride.from_proto(data_source.kafka_options.spark_override),
        )
    else:
        raise ValueError("Could not identify the source type being added")



import enum


class ValueType(enum.Enum):
    """
    Feature value type. Used to define data types in Feature Tables.
    """

    UNKNOWN = 0
    BYTES = 1
    STRING = 2
    INT32 = 3
    INT64 = 4
    DOUBLE = 5
    FLOAT = 6
    BOOL = 7
    BYTES_LIST = 11
    STRING_LIST = 12
    INT32_LIST = 13
    INT64_LIST = 14
    DOUBLE_LIST = 15
    FLOAT_LIST = 16
    BOOL_LIST = 17



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: feast/core/DataFormat.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1b\x66\x65\x61st/core/DataFormat.proto\x12\nfeast.core\"g\n\nFileFormat\x12>\n\x0eparquet_format\x18\x01 \x01(\x0b\x32$.feast.core.FileFormat.ParquetFormatH\x00\x1a\x0f\n\rParquetFormatB\x08\n\x06\x66ormat\"\xd8\x01\n\x0cStreamFormat\x12:\n\x0b\x61vro_format\x18\x01 \x01(\x0b\x32#.feast.core.StreamFormat.AvroFormatH\x00\x12<\n\x0cproto_format\x18\x02 \x01(\x0b\x32$.feast.core.StreamFormat.ProtoFormatH\x00\x1a!\n\x0bProtoFormat\x12\x12\n\nclass_path\x18\x01 \x01(\t\x1a!\n\nAvroFormat\x12\x13\n\x0bschema_json\x18\x01 \x01(\tB\x08\n\x06\x66ormatB{\n\x1e\x64\x65v.caraml.store.protobuf.coreB\x0f\x44\x61taFormatProtoZHgithub.com/caraml-dev/caraml-store/caraml-store-sdk/go/protos/feast/coreb\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'feast.core.DataFormat_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\036dev.caraml.store.protobuf.coreB\017DataFormatProtoZHgithub.com/caraml-dev/caraml-store/caraml-store-sdk/go/protos/feast/core'
  _FILEFORMAT._serialized_start=43
  _FILEFORMAT._serialized_end=146
  _FILEFORMAT_PARQUETFORMAT._serialized_start=121
  _FILEFORMAT_PARQUETFORMAT._serialized_end=136
  _STREAMFORMAT._serialized_start=149
  _STREAMFORMAT._serialized_end=365
  _STREAMFORMAT_PROTOFORMAT._serialized_start=287
  _STREAMFORMAT_PROTOFORMAT._serialized_end=320
  _STREAMFORMAT_AVROFORMAT._serialized_start=322
  _STREAMFORMAT_AVROFORMAT._serialized_end=355
# @@protoc_insertion_point(module_scope)



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import collections.abc
import feast.types.Value_pb2
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import sys

if sys.version_info >= (3, 8):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing_extensions.final
class FeatureSpec(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing_extensions.final
    class LabelsEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        value: builtins.str
        def __init__(
            self,
            *,
            key: builtins.str = ...,
            value: builtins.str = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal["key", b"key", "value", b"value"]) -> None: ...

    NAME_FIELD_NUMBER: builtins.int
    VALUE_TYPE_FIELD_NUMBER: builtins.int
    LABELS_FIELD_NUMBER: builtins.int
    name: builtins.str
    """Name of the feature. Not updatable."""
    value_type: feast.types.Value_pb2.ValueType.Enum.ValueType
    """Value type of the feature. Not updatable."""
    @property
    def labels(self) -> google.protobuf.internal.containers.ScalarMap[builtins.str, builtins.str]:
        """Labels for user defined metadata on a feature"""
    def __init__(
        self,
        *,
        name: builtins.str = ...,
        value_type: feast.types.Value_pb2.ValueType.Enum.ValueType = ...,
        labels: collections.abc.Mapping[builtins.str, builtins.str] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["labels", b"labels", "name", b"name", "value_type", b"value_type"]) -> None: ...

global___FeatureSpec = FeatureSpec
